;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe89f86111d87f7683856d42c30977f52 . "./code.cg2js/cg-common-control.fasl")
(#xbdce6c27521b9d790409a0925a0c0439 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x4c5eb7ab73dce83a33ca005a093a0bcf . "./code.cg2js/cg-yes-no-string.fasl")
(#xa7f90f65a2663b61823f7d55fab35611 . "./code.cg2js/cg-clipboard.fasl")
(#x35204e92ad58c58838dbc3cd5db6ee91 . "./code.cg2js/cg-dde-utils.fasl")
(#xd496ebdadbef8a2537f1bea4ad5445c9 . "./code.cg2js/cg-editable-text.fasl")
(#xf1b67c2ac95cf9b3fa1ed3b21e9a6455 . "./code.cg2js/cg-player.fasl")
(#x1439d011d0cee07eb0b429f2afff8e4c . "./code.cg2js/cg-text-or-combo.fasl")
(#x734c6589ea33232988721b94bf9d9139 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x2188db351e6c41c1d4f7387c6e370f24 . "./code.cg2js/cg-graphics-context.fasl")
(#x1007c48e2ab1085d45be7be20f5e9e57 . "./code.cg2js/cg-choose-printer.fasl")
(#x92243221f1583a3d0fd2d2cce207e085 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe4c56bd09cb0b79ec2602f5461fbcded . "./code.cg2js/cg-edit-in-place.fasl")
(#xd57485e4dc6daaa6fcf4613704852a6d . "./code.cg2js/cg-split-bar.fasl")
(#x134420761bd6ec322fb3c0c4864c29ce . "./code.cg2js/cg-timer.fasl")
(#xda068c34351daa9e59c01a2fd7823173 . "./code.cg2js/cg-progress-indicator.fasl")
(#xffe85e109d6a7c193595338a5ecc1b6b . "./code.cg2js/cg-object-editor.fasl")
(#xb07d1c71c0c4f3acb23bcedd5ef602fb . "./code.cg2js/cg-fill-texture.fasl")
(#x32906f96878bb9b7c485fad3b98e3a78 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x357f7c4dec109b661b4b330e21c4355a . "./code.cg2js/cg-os-window.fasl")
(#x3613138314a92c505d33077c344311ef . "./code.cg2js/cg-color-dialog.fasl")
(#x552727604d15ab2223bbcb6fa9e1fa37 . "./code.cg2js/cg-combo-box.fasl")
(#x74479799838a20a932b60205920c4155 . "./code.cg2js/cg-directory-dialog.fasl")
(#x7e363098fc963d2f305f30bf8ad9ab44 . "./code.cg2js/cg-tooltip.fasl")
(#x8e2d58df3df290093adbbdf2685cccaa . "./code.cg2js/cg-lettered-menu.fasl")
(#x8c9d7fce417b1f52beec97982e5ff84a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x48b4732413c1e18581048dac2e7496f1 . "./code.cg2js/cg-pixmap.fasl")
(#xf93f6c5a14f7c7f791f88c9e25341338 . "./code.cg2js/cg.fasl")
(#x26aeee61d4558fcd1305af909cb2ed61 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x7f4e0e8147e3f1471eb14716aa82bab3 . "./code.cg2js/cg-template-string.fasl")
(#x7fcf973afb5b075aaa6ce9d0166472d5 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x98fc1bc258fea543efbb08607b2caf12 . "./code.cg2js/cg-check-box.fasl")
(#x0e2de4e69e0043efef788e85725516ae . "./code.cg2js/cg-toggling-widget.fasl")
(#xc0ab909c384dc78e8e1b0f8493aa0430 . "./code.cg2js/cg-radio-button.fasl")
(#xf6b1921074eddfca6333af4bbb839dd4 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x1a4fc15b9e33217a2a696b8ea349bc0f . "./code.cg2js/cg-scaling-stream.fasl")
(#x4523c7e17a3c2418bb8522499b3aaec2 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x040b6d398ab8d653bcd7c5f4fd0e9f1f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x6ff2d5e109cd621da1cc6847d1af3415 . "./code.cg2js/cg-grid-widget.fasl")
(#x1e02b6969cc52f5a6c17fb8299933aa6 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xdebe107e24c8e5effadfec4ea4892975 . "./code.cg2js/cg-item-list.fasl")
(#x443b46c12f136e5174aed88d1cda0dcc . "./code.cg2js/cg-paren-matching.fasl")
(#xecba26d2fc929ffef1b93843337bf7b6 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9771e52daa1d112c50ac8c5ed770e90f . "./code.cg2js/cg-html-dialog.fasl")
(#xa37336a4373363ddbd0d339dd1e9d9c7 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xc397c52d519960ddbe18e90d35b9cc08 . "./code.cg2js/cg-project-window.fasl")
(#x4714b61429ec324df018ddf77e4e45f9 . "./code.cg2js/cg-toolbar.fasl")
(#x8be3e0ee066cd84fda37dc47fe7f8bbf . "./code.cg2js/cg-caret.fasl")
(#xd49756fbd464acaecbf6bb392b5e5851 . "./code.cg2js/cg-scroll-bar.fasl")
(#x3cc0dff79a4d7e5491f915fe6a016318 . "./code.cg2js/cg-web-browser.fasl")
(#xdcbc3cce427ec6b45c1e5f9ed7f0d694 . "./code.cg2js/cg-dropping-outline.fasl")
(#x8070df7ad11a4fed43043af36de67216 . "./code.cg2js/cg-font-dialog.fasl")
(#x3f498d0bbba78c962c22ad9c0cedfb68 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xa61681ab68410f3336876658c5f9ead4 . "./code.cg2js/cg-yes-no-list.fasl")
(#x6fc284995c6aa55908704493189a0a27 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xd7c186c809a3e13f198a1f409f079bfb . "./code.cg2js/cg-lamp.fasl")
(#x931afb06fe0d07e63765c42b3c06e498 . "./code.cg2js/cg-drawable.fasl")
(#x85ac58b48c390169be83fdde49da52a1 . "./code.cg2js/cg-choice-list.fasl")
(#x4eaf8b6edd0292721b9ae2e2efd1b567 . "./code.cg2js/cg-curve.fasl")
(#x5ee60e672d1c7f1bed5469d64d4e1d43 . "./code.cg2js/cg-base.fasl")
(#x581d3ee16f002c00763b00dba967dfad . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xfd30086ec551f649e27a6d1d609c055d . "./code.cg2js/cg-printing.fasl")
(#x72bf6ecb6e5c1ce1e2deac2dd8b5c709 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x6e5fda568274df348db88eb85821d2a9 . "./code.cg2js/cg-hotspot.fasl")
(#x562e919afe0550e513bc1efca3efa6c4 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xbf8f21e6978f10ff756e9fe39b9a7383 . "./code.cg2js/cg-utility-dialog.fasl")
(#x4aad015646f684bd9bca185a15d12a4e . "./code.cg2js/cg-shortcut-menu.fasl")
(#x97dc533db7804e460fbab2f47b0d4667 . "./code.cg2js/cg-comtab.fasl")
(#x142fa4955009e6e5f4483a885d08dea3 . "./code.cg2js/cg-os-widget.fasl")
(#x99c5800ff22c2e9a6ae0071d8c07aceb . "./code.cg2js/cg-icon.fasl")
(#xbaaac5730b202e073972c28c9f16bf43 . "./code.cg2js/cg-dev.fasl")
(#x9d6b01be3514f9d360f63d15112625f6 . "./code.cg2js/cg-calendar.fasl")
(#x2e37ef99f64360932f19d696b5d5cd33 . "./code.cg2js/cg-group-box.fasl")
(#x78a29bebe9ae077c0f773178e6f612c1 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x847e7f88fd5ca0451d54605ff4b46128 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x1c2a01b567abd28b312ccd71026f3362 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x87bb19f18611b4920e096a134f4d92bc . "./code.cg2js/cg-nodes.fasl")
(#x4afb25ac8f363ab8716617705021ba2f . "./code.cg2js/cg-intersect.fasl")
(#x189bb47e794bb0ab9a41d7def6ceb14b . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x4126021676347fb47699a9d6a9b3b562 . "./code.cg2js/cg-class-support.fasl")
(#xa842f35bb07c5ffbbe176b836c71b854 . "./code.cg2js/cg-dialog-item.fasl")
(#x569d15465ca2078d8154627838a29c5f . "./code.cg2js/cg-plot-widget.fasl")
(#xbee001fa93a2a8bae205b440127e6eec . "./code.cg2js/cg-pixmap-printing.fasl")
(#x275bcfeea027cd2a8b418acc03994e3c . "./code.cg2js/cg-up-down-control.fasl")
(#xb3a416ecc26a036a51ae71b0dd10a150 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc32c7a2482aa7644e236e6a1806ad88d . "./code.cg2js/cg-status-bar.fasl")
(#x8f0fb0918bb70d0a63864a0bfa4b0635 . "./code.cg2js/cg-class-grid.fasl")
(#xc21d07a8601eeac309d49d1dce189705 . "./code.cg2js/cg-wrap-string.fasl")
(#x517f7d57788a31208f75b4d184767ca0 . "./code.cg2js/cg-layout.fasl")
(#x402d9e0abb4c83a32232b929d79050ef . "./code.cg2js/cg-lisp-widget.fasl")
(#xd67c9b790d477754eed11227d811dc49 . "./code.cg2js/cg-button.fasl")
(#xead83480c570f0c52b781d08c9feaa90 . "./code.cg2js/cg-string-dialog.fasl")
(#xad2604a9b747b423d1d4678e967b0bc2 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x5bafd8be5db00a649de5bf30dd83e502 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9efff49cdbc65dffb11f8e92fe5da933 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc2aa6e03e040c5701ec467713ba765f8 . "./code.cg2js/cg-static-text.fasl")
(#x5a2d56f8941abe53b20fd3665bd1459c . "./code.cg2js/cg-outline.fasl")
(#xc8a4581c88c32b1b06fa43c256d9eeba . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7ff51c65bb6f32a5668951001fd4c5b4 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6c3c32de17797e004e28ef7d6b75b2f6 . "./code.cg2js/cg-picture-widget.fasl")
(#x4fba1b1ef37f884d26e68ed6066b7197 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x169b79de9b34c15e2cf26126d66bf292 . "./code.cg2js/cg-chart-widget.fasl")
(#x2ea78c0f05a557e308690ac45b9847cb . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x1c7d928493af55818110ec7dc49ca44e . "./code.cg2js/cg-lisp-text.fasl")
(#x6473a8f994367f842e8673009dc1a02e . "./code.cg2js/cg-palette.fasl")
(#xc5de3b12b165b834e5e6e2d864231467 . "./code.cg2js/cg-property.fasl")
(#x334ed16ed7421b92d45ef56d0261a921 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xbbebcde1db24bde86daaca24f1c445a1 . "./code.cg2js/cg-file-dialog.fasl")
(#x63ad9ae850fa3d7dcae67ca02775cd49 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x8e9fd1bb10225c150d553652f2e2f9d2 . "./code.cg2js/cg-get-pixmap.fasl")
(#xc48752ecb0a19c54df5dda206ace130c . "./code.cg2js/cg-tray.fasl")
(#xd351e96be5bad3e312626d95d072b6d7 . "./code.cg2js/cg-acache.fasl")
(#x129a76807665c1e318599f3ad8f442a2 . "./code.cg2js/cg-message-dialog.fasl")
(#xc1f81532628f4cb64c5044e90666843f . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xabd514ac3c5b6d6a197b4384b8ad2d25 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf7756d2bd1494dca3eb5ba7c2266517e . "./code.cg2js/cg-tab-control.fasl")
(#xc8118a5fe4d1bcc620714f17a2e61a2a . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xbb7d5d72405e9699e18a27cf60a5951d . "./code.cg2js/cg-menu-tooltip.fasl")
(#xce70cafc7414f5f95c7ac1505325f089 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2340cd2dc608282c3117903c6fb5c4c6 . "./code.cg2js/cg-menu.fasl")
(#x5c76f77c9bbf50b89ced07547d17025f . "./code.cg2js/cg-get-position.fasl")
(#x7f283b9c65312d3c37f2c5c0fcc2d8f3 . "./code.cg2js/cg-trackbar.fasl")
(#x62b58b26a4cfe18db71deb8c44eea21a . "./code.cg2js/cg-selected-object.fasl")
(#x2e686d0ff0ce468a9b52fa1d753a29a8 . "./code.cg2js/cg-list-view.fasl")
(#xf09dae141369ef90dab7f757e2eac3c3 . "./code.cg2js/cg-html-widget.fasl")
(#xd71113d5fa43d73410aa4dd12725d3dc . "./code.cg2js/cg-sample-file-menu.fasl")
(#x96c5ec224203cdf966c7ed5b981a5d53 . "./code.cg2js/cg-text-widget.fasl")
(#x17de14043156e764cf0b719e2e466e28 . "./code.idejs/ide-base.fasl")
(#x0afe874e5392d1a7bfe268801ed2d081 . "./code.idejs/ide-process-browser.fasl")
(#xcad672bb39c5f1bdc49d5e00162dbe9c . "./code.idejs/ide-patch-dialog.fasl")
(#xd6c1197edadf8a0343939d1a3f53e709 . "./code.idejs/ide-keyshort.fasl")
(#x0d23c9df8135ae32a63f3eed572a0d87 . "./code.idejs/ide-window-list.fasl")
(#x5839e38005aab62965b24b092f82185a . "./code.idejs/ide.fasl")
(#x4ef94645ff70729f08219ff6c5fa1321 . "./code.idejs/ide-editor.fasl")
(#xe1bc00ac45ccc3356326be389f4c933d . "./code.idejs/ide-clipboard.fasl")
(#xeda1f22eb6ddd34d78581a05d401460e . "./code.idejs/ide-trace.fasl")
(#x4c7d2a2503d0e9e5e5da3ff642ada47c . "./code.idejs/ide-class-browser.fasl")
(#x10ec8022ed06be022c566ae490ecd30b . "./code.idejs/ide-grapher.fasl")
(#x867ebf633e89eaf358c86996a05fbdd7 . "./code.idejs/ide-apropos.fasl")
(#x6754630c51b17fc1bffb536c5915ad0f . "./code.idejs/ide-navigator.fasl")
(#x047390f686cc67fe2e099620d157f0bd . "./code.idejs/ide-find-definitions.fasl")
(#x9d0f37a292439d7075116ccbc2e06f56 . "./code.idejs/ide-project.fasl")
(#xade5ea6209cc67e7c41e9885c2a58001 . "./code.idejs/ide-class-grapher.fasl")
(#x48a231ebd04ee4ce5aa40d1edb5a21c7 . "./code.idejs/ide-acache.fasl")
(#x75a27746d01551be3da87d7bbd39b96f . "./code.idejs/ide-find-in-files.fasl")
(#x1ca9e701c46fa48dd2882f87ac935a39 . "./code.idejs/ide-inspector.fasl")
(#x313d31bccd9123f2b41df930d177bd44 . "./code.idejs/ide-intro.fasl")
(#x898df82c8247d0920a38e2c9821c081f . "./code.idejs/ide-package-list.fasl")
(#xcb403d440507c292484d83fb17d5b855 . "./code.idejs/ide-dev.fasl")
(#x9a7bde47a3860f167eee9621de5713b0 . "./code.idejs/ide-form.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
